#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Language Identification Model

Don't forget to comment line below at load_model function in FastText.py to silence warning
    eprint("Warning : `load_model` does not return WordVectorModel
"""

import os
import re
from typing import Tuple, Optional

from trtopicter.detectors import _load_model
from trtopicter.exceptions import MLModelNotExistError, MLModelFileBrokenError

__all__ = ['LanguageDetector']
__version__ = '0.0.0.1'
__author__ = 'Apdullah Yayık <apdullah.yayik@gmail.com>'
__since__ = '25.11.2020'


class LanguageDetector:
    __slots__ = ['probability_threshold', 'model']

    def __init__(self, ml_model_path: str, probability_threshold: float):
        probability_threshold = min(probability_threshold, 1)
        self.probability_threshold = max(probability_threshold, 0)

        if not os.path.isfile(ml_model_path):
            raise MLModelNotExistError(ml_model_path)

        try:
            self.model = _load_model(ml_model_path)
        except Exception:
            raise MLModelFileBrokenError(ml_model_path)

    @staticmethod
    def __clean(text: str) -> str:
        # remove punctuation
        from string import punctuation
        text = text.translate(str.maketrans('', '', punctuation))

        # remove numbers
        text = re.sub(r"\d", "", text)

        # remove whitespaces and return lower-case
        text = re.sub(r'[\n\t]', ' ', text)
        return re.sub(r' +', ' ', text.lower())

    def predict_language(self, text: str, number_of_output: Optional[int] = None,
                         reduction_fold: Optional[int] = None) -> Optional[dict]:
        if number_of_output is None:
            number_of_output = 1
        if reduction_fold is None:
            reduction_fold = 1
        probability_threshold = self.probability_threshold / reduction_fold
        if not text.strip():
            prediction = ([], ())
        else:
            try:
                prediction = self.model.predict(self.__clean(text),
                                                k=number_of_output,
                                                threshold=probability_threshold)
            except TypeError:
                prediction = ([], ())
        return self.__format_prediction(prediction, probability_threshold)

    @staticmethod
    def __format_prediction(prediction: Tuple, probability_threshold: float) -> Optional[dict]:
        label, probability = prediction
        if len(label) != 1 or len(probability) != 1:
            return None
        if isinstance(probability[0], float):
            probability_value = probability[0]
            if probability_value > probability_threshold:
                if isinstance(label[0], str):
                    label_regex = re.match(pattern=r'__label__([a-z]{2,3})', string=label[0])
                    if label_regex:
                        return {'label': label_regex.group(1), 'probability': probability_value}
                    else:
                        return None
                else:
                    return None
            else:
                return None
        else:
            return None

    def __repr__(self):
        return '<LanguageDetector> object dedicated to load machine learning models and provide predictions'



import os
import re
from string import punctuation
from typing import Optional, Tuple

from trtopicter.detectors import _load_model
from trtopicter.exceptions import MLModelNotExistError, MLModelFileBrokenError
from trtopicter.tokenizer import word_tokenize

__all__ = ['DomainDetector']
__version__ = '0.0.0.1'
__author__ = 'Apdullah Yayık <apdullah.yayik@gmail.com>'
__since__ = '25.11.2020'

TR_TO_ENG = {
    'siyaset': 'politics',
    'dunya': 'world',
    'ekonomi': 'economy',
    'saglik': 'health',
    'spor': 'sport',
    'teknoloji': 'technology',
    'kultur': 'culture',
    'religion': 'religion',
    'education': 'education',
}


class UnicodeTr(str):
    CHAR_MAP = {
        "to_lower": {
            u"I": u"ı",
            u"İ": u"i",
        }
    }

    def lower(self):
        for key, value in self.CHAR_MAP['to_lower'].items():
            self = self.replace(key, value)
        return self.lower()


class DomainDetector:
    __slots__ = ('stop_words', 'word_tokenizer_pre_compiled_regex', 'probability_threshold', 'model')

    def __init__(self, ml_model_path: str,
                 probability_threshold: float,
                 word_tokenizer_pre_compiled_regex,
                 stop_words=None):
        self.stop_words = stop_words
        self.word_tokenizer_pre_compiled_regex = word_tokenizer_pre_compiled_regex
        probability_threshold = min(probability_threshold, 1)
        self.probability_threshold = max(probability_threshold, 0)

        if not os.path.isfile(ml_model_path):
            raise MLModelNotExistError(ml_model_path)

        try:
            self.model = _load_model(ml_model_path)
        except Exception:
            raise MLModelFileBrokenError(ml_model_path)

    def __clean(self, line: str):

        # normalization (lower case, punctuation, numbers, white space)
        line = line.translate(str.maketrans('', '', punctuation))
        line = re.sub(r'( +)|([\d\n])', ' ', UnicodeTr(line).lower().strip())

        # stop words removal
        if self.stop_words:
            line_words = word_tokenize(line, self.word_tokenizer_pre_compiled_regex)
            line_words = [word for word in line_words if word not in self.stop_words]
            return ' '.join(line_words)
        else:
            return line

    def predict_domain(self, text: str, character_limit: int, number_of_output: Optional[int] = None) -> Optional[dict]:

        if number_of_output is None:
            number_of_output = 1
        clean_text = self.__clean(text)
        if not len(clean_text) > character_limit * 0.8:
            prediction = ([], ())
        else:
            try:
                prediction = self.model.predict(clean_text,
                                                k=number_of_output,
                                                threshold=self.probability_threshold)
            except TypeError:
                prediction = ([], ())

        return self.__format_prediction(prediction, self.probability_threshold)

    @staticmethod
    def __format_prediction(prediction: Tuple, probability_threshold: float) -> Optional[dict]:
        label, probability = prediction
        if len(label) != 1 or len(probability) != 1:
            return None
        if isinstance(probability[0], float):
            probability_value = probability[0]
            if probability_value > probability_threshold:
                if isinstance(label[0], str):
                    label_regex = re.match(pattern=r'__label__([a-z]*)', string=label[0])
                    if label_regex:
                        probability_value = min(1.00, probability_value)
                        return {'label': TR_TO_ENG[label_regex.group(1)],
                                'probability': probability_value}
                    else:
                        return None
                else:
                    return None
            else:
                return None
        else:
            return None

    def __repr__(self):
        return '<DomainDetector> object dedicated to load machine learning models and provide predictions'



#!/usr/bin/env python.
# -*- coding: utf-8 -*-

"""
Word tokenizer
"""

__all__ = ['word_tokenize', 'compile_word_tokenizer_regex']
__since__ = '25.11.2020'
__version__ = '0.0.0.1'
__author__ = 'Apdullah Yayık <apdullahyayik@gmail.com>'

from typing import Tuple, Union, List

import re


def compile_word_tokenizer_regex():
    suffixes = r"[a-zğçşöüı]{3,}' ?[a-zğçşöüı]+"
    numbers = r"%\d{2,}[.,:/\d-]+"
    any_word = r"[a-zğçşöüı_+%\.()@&`’/\\\d-]+"
    punctuations = r"[a-zğçşöüı]*[,!?;:]"

    return re.compile(
        "|".join(
            [suffixes,
             numbers,
             any_word,
             punctuations
             ]
        ), re.I
    )


def word_tokenize(sentence: str, word_regex) -> Tuple:
    try:
        words: Union[List] = word_regex.findall(sentence)
    except (re.error, TypeError):
        return ()
    else:
        # If last word ends with dot, it should be another word
        words: Union[Tuple] = tuple(words)
        if words:
            end_dots = re.search(r'\b(\.+)$', words[-1])
            if end_dots:
                dots: str = end_dots.group(1)
                words = words[:-1] + (words[-1][:-len(dots)],) + (dots,)
        return words



import os
import re
from json import load

from trtopicter.detectors.domain_detector import DomainDetector
from trtopicter.detectors.language_detector import LanguageDetector
from trtopicter.exceptions import NonTurkishStringError
from trtopicter.tokenizer import compile_word_tokenizer_regex

__all__ = ['TrTopicter']
__version__ = '0.0.0.1'
__author__ = 'Apdullah Yayık <apdullah.yayik@gmail.com>'
__since__ = '25.11.2020'


class TrTopicter:
    __slots__ = ('language_detection_model', 'domain_detection_model', 'text', 'parameters')

    def __init__(self):
        this_directory = os.path.abspath(os.path.dirname(__file__))
        self.parameters = self.__read_params(os.path.join(this_directory,'./configuration.json'))
        language_probability_threshold = self.parameters['LANGUAGE_IDENTIFICATION']['probability_threshold']
        self.language_detection_model = LanguageDetector(os.path.join(this_directory,'models/language/lid.176.ftz'),
                                                         language_probability_threshold)
        domain_probability_threshold = self.parameters['DOMAIN_DETECTION']['probability_threshold']
        word_tokenizer_pre_compiled_regex = compile_word_tokenizer_regex()
        stop_words = self.__read_stop_words(os.path.join(this_directory,'stop_words/tr_stop_words'))
        self.domain_detection_model = DomainDetector(os.path.join(this_directory,'models/domain_detector/tr_domain_data.lite'),
                                                     domain_probability_threshold,
                                                     word_tokenizer_pre_compiled_regex,
                                                     stop_words)

    @staticmethod
    def __read_stop_words(stop_word_file_path):
        stop_words = set()
        with open(stop_word_file_path, 'r', encoding='utf-8') as fp:
            for line in fp:
                stop_words.add(re.sub(r'\n', '', line))
        return stop_words

    @staticmethod
    def __read_params(params: str):
        with open(params, 'r', encoding='utf-8') as f:
            parameters = load(f)
        return parameters

    def get_topic(self, text):
        if not isinstance(text, str):
            raise TypeError('text should be given as string')
        language_info = self.language_detection_model.predict_language(text)

        if language_info:
            if language_info['label'] == 'tr':
                domain_info = self.domain_detection_model.predict_domain(
                    text, self.parameters['DOMAIN_DETECTION']['limit']['character'] * 0.8
                )
                if domain_info:
                    return domain_info
                else:
                    return None
        raise NonTurkishStringError

    def __repr__(self):
        return '<TrTopicter> object dedicated to load machine learning models and provide predictions'



#!/usr/bin/env python.
# -*- coding: utf-8 -*-

"""
Custom exceptions
"""

import inspect
import os

__all__ = ['short_descriptions', 'TrTopicterError', 'MLModelFileBrokenError', 'NonTurkishStringError',
           'MLModelNotExistError', 'get_line_info']
__version__ = '0.0.0.1'
__since__ = '25.11.2020'
__author__ = 'Apdullah Yayık <apdullahyayik@gmail.com>'

from typing import Union

short_descriptions = {
    1: "ML Model not exists, get request",
    2: "ML model file broken error, get request latest one",
    3: "Non-Turkish document is not supported"
}


class TrTopicterError(Exception):
    """
    Main project-based exception class
    """
    __slots__ = ['meta_data']

    def __init__(self, internal_detail: any = None):
        self.meta_data = get_line_info()
        self.meta_data['internal_detail'] = str(internal_detail)


class MLModelNotExistError(TrTopicterError):
    def __init__(self, internal_detail: any = None):
        super().__init__(internal_detail)
        self.meta_data['code'] = 1
        self.meta_data['short_description'] = short_descriptions[self.meta_data['code']]


class MLModelFileBrokenError(TrTopicterError):
    def __init__(self, internal_detail: any = None):
        super().__init__(internal_detail)
        self.meta_data['code'] = 2
        self.meta_data['short_description'] = short_descriptions[self.meta_data['code']]


class NonTurkishStringError(TrTopicterError):
    def __init__(self, internal_detail: any = None):
        super().__init__(internal_detail)
        self.meta_data['code'] = 3
        self.meta_data['short_description'] = short_descriptions[self.meta_data['code']]


def get_line_info(usage='exception') -> Union[str, dict]:
    frame = inspect.currentframe()
    if usage == 'logger':
        information = inspect.getframeinfo(frame.f_back)
        return ' - '.join(
            [
                os.path.basename(information.filename),
                information.function,
                str(information.lineno),
            ]
        )
    else:
        information = inspect.getframeinfo(frame.f_back.f_back.f_back.f_back)
        return {'line_number': information.lineno,
                'file_name': os.path.basename(information.filename),
                'function_name': information.function}



from distutils.core import setup

from os import path
from setuptools import find_packages

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md')) as f:
    long_description = f.read()

setup(
    name='trtopicter',
    # packages=['trtopicter'],
    packages=find_packages(),
    include_package_data=True,
    version='0.0.4',
    license='MIT',
    description='Turkish topic detector using machine learning',
    # long_description_content_type='text/markdown',
    # long_description=long_description,
    author='Apdullah YAYIK',
    author_email='apdullahyayik@gmail.com',
    url='https://github.com/apdullahyayik/Tr-topicter',
    download_url='https://github.com/apdullahyayik/Tr-topicter/archive/v0.0.3.tar.gz',
    keywords=['Turkish domain detector', 'text classification', 'topic classification'],
    install_requires=['fasttext'],
    python_requires='>=3.4'
)
